﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail.Test
{
    /// <summary>
    ///    Orderに対するユニット・テストです。
    /// </summary>
    [TestClass]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1001")]
    public class OrderUnitTest
    {
        /// <summary>
        ///    テストに使用する型付DataSet。
        /// </summary>
        private RetailDataSet dataSet;

        /// <summary>
        ///    テストに使用する型付DataTable。
        /// </summary>
        private RetailDataSet.OrderDataTable dataTable;

        /// <summary>
        ///    利用者のGUID（user1）。
        /// </summary>
        private Guid userInformationNumber = new Guid("25b60925-a865-4ca8-90fe-6c6ed493e851");

        /// <summary>
        ///    環境を初期化します。
        /// </summary>
        [TestInitialize]
        public void TestInitialize()
        {
            //  フィールドを初期化します。
            dataSet = new RetailDataSet();
            dataTable = dataSet.Order;

            //  データベースを初期化します。
            DataBaseMaintainer.Initialize();
        }

        /// <summary>
        ///    環境をクリーンアップします。
        /// </summary>
        [TestCleanup]
        public void TestCleanup()
        {
            //  フィールドをクリーンアップします。
            dataSet.Dispose();

            //  データベースをクリーンアップします。
            DataBaseMaintainer.Cleanup();
        }

        /// <summary>
        ///    MakeOrder()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void MakeOrder()
        {
            {
                //  必要な情報を収集。
                UserInformation.Fill(dataSet);
                Item.Fill(dataSet);

                //  注文を作成。
                dataSet.Order.AddOrderRow(
                    "xxx0000",
                    dataSet.UserInformation.FindByUserInformationNumber(userInformationNumber),
                    "000-0002",
                    "群馬県桐生市川内町",
                    "2-2-2",
                    "0277-22-0002",
                    "利用者 太郎",
                    string.Empty,
                    "VISA",
                    "0123456789",
                    12,
                    2007);

                //  注文明細を作成。
                dataSet.OrderDetail.AddOrderDetailRow(
                    1L,
                    dataSet.Order[0],
                    dataSet.Item[0],
                    dataSet.Item[0].Price,
                    1);
                dataSet.OrderDetail.AddOrderDetailRow(
                    2L,
                    dataSet.Order[0],
                    dataSet.Item[1],
                    dataSet.Item[1].Price * 2,
                    2);

                //  注文。
                Order.MakeOrder(dataSet);

                //  エラーがないことを確認。
                Assert.IsFalse(dataSet.HasErrors);
            }

            dataSet.Clear();

            {
                //  必要な情報を収集。
                UserInformation.Fill(dataSet);
                Item.Fill(dataSet);

                //  注文を作成。
                dataSet.Order.AddOrderRow(
                    "xxx0000",
                    dataSet.UserInformation.FindByUserInformationNumber(userInformationNumber),
                    "000-0002",
                    "群馬県桐生市川内町",
                    "2-2-2",
                    "0277-22-0002",
                    "利用者 太郎",
                    string.Empty,
                    "VISA",
                    "0123456789",
                    12,
                    2007);

                //  注文明細を作成。
                dataSet.OrderDetail.AddOrderDetailRow(
                    1L,
                    dataSet.Order[0],
                    dataSet.Item[0],
                    dataSet.Item[0].Price,
                    9);

                //  注文。
                Order.MakeOrder(dataSet);

                //  エラーが発生することを確認。
                Assert.AreEqual("申し訳ありません。エンゼル・フィッシュ（大）は、数量8しか在庫がありません。", dataSet.OrderDetail[0].GetColumnError(dataSet.OrderDetail.QuantityColumn));
            }
        }

        /// <summary>
        ///    GetTotalPrice()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void GetTotalPrice()
        {
            OrderDetail.Fill(dataSet);
            Assert.AreEqual(4950M, Order.GetTotalPrice(dataSet, dataSet.Order[0].OrderNumber));
        }

        /// <summary>
        ///    入力値検証をテストします。
        /// </summary>
        [TestMethod]
        public void Validate()
        {
            //  追加よりも変更の方が楽なので、まずはデータを取得します。
            Order.Fill(dataSet);

            //  不正な入力値を作成。それぞれが独立であることが分かっているので、組み合わせのテストはしません。
            dataTable[0].ShippingZipCode = string.Empty;
            dataTable[1].ShippingZipCode = "x000-0000x";
            dataTable[0].ShippingAddress1 = string.Empty;
            dataTable[0].ShippingPhoneNumber = string.Empty;
            dataTable[1].ShippingPhoneNumber = "x00-0000-0000x";
            dataTable[0].CreditCardChargeNumber = string.Empty;
            foreach (RetailDataSet.OrderDetailRow dr in dataTable[0].GetOrderDetailRows())
            {
                dr.Delete();
            }

            //  Update()メソッドを呼びます。Validatorが呼び出され、入力値検証が実行されます。
            Order.Update(dataSet);

            //  正しくエラーが設定されたか検証します。
            Assert.AreEqual("出荷先郵便番号は必須入力です。", dataTable[0].GetColumnError(dataTable.ShippingZipCodeColumn));
            Assert.AreEqual("出荷先郵便番号は半角数字3桁と「-（半角ハイフン）」、半角数字4桁で入力してください。", dataTable[1].GetColumnError(dataTable.ShippingZipCodeColumn));
            Assert.AreEqual("出荷先住所1は必須入力です。", dataTable[0].GetColumnError(dataTable.ShippingAddress1Column));
            Assert.AreEqual("出荷先電話番号は必須入力です。", dataTable[0].GetColumnError(dataTable.ShippingPhoneNumberColumn));
            Assert.AreEqual("出荷先電話番号は半角数字および「-（半角ハイフン）」で入力してください。", dataTable[1].GetColumnError(dataTable.ShippingPhoneNumberColumn));
            Assert.AreEqual("クレジット・カードを承認できませんでした。クレジット・カード情報を確認してください。", dataTable[0].GetColumnError(dataTable.CreditCardChargeNumberColumn));
            Assert.AreEqual("注文には最低一つの注文明細が必要です。", dataTable[0].RowError);
        }

        /// <summary>
        ///    Fill()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void Fill()
        {
            //  Fill()メソッドを呼び出せることを確認します。
            Order.Fill(dataSet);
        }

        /// <summary>
        ///    FillWithChildren()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillWithChildren()
        {
            //  FillWithChildren()メソッドを呼び出せることを確認します。
            Order.FillWithChildren(dataSet);
        }

        /// <summary>
        ///    GetData()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void GetData()
        {
            //  GetData()メソッドを呼び出せることを確認します。
            Order.GetData();
        }
    }
}
