﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail.Test
{
    /// <summary>
    ///    ReservedItemに対するユニット・テストです。
    /// </summary>
    [TestClass]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1001")]
    public class ReservedItemUnitTest
    {
        /// <summary>
        ///    テストに使用する型付DataSet。
        /// </summary>
        private RetailDataSet dataSet;

        /// <summary>
        ///    テストに使用する型付DataTable。
        /// </summary>
        private RetailDataSet.ReservedItemDataTable dataTable;

        /// <summary>
        ///    利用者のGUIDその1（administrator）。
        /// </summary>
        private Guid userInformationNumber1 = new Guid("673987c3-1ced-4ce0-9507-e1d4d2c281e2");

        /// <summary>
        ///    利用者のGUIDその2（user1）。
        /// </summary>
        private Guid userInformationNumber2 = new Guid("25b60925-a865-4ca8-90fe-6c6ed493e851");

        /// <summary>
        ///    利用者のGUIDその3（user2）。
        /// </summary>
        private Guid userInformationNumber3 = new Guid("9711718d-95b7-4032-bfc0-16db18fb9996");

        /// <summary>
        ///    環境を初期化します。
        /// </summary>
        [TestInitialize]
        public void TestInitialize()
        {
            //  フィールドを初期化します。
            dataSet = new RetailDataSet();
            dataTable = dataSet.ReservedItem;

            //  データベースを初期化します。
            DataBaseMaintainer.Initialize();
        }

        /// <summary>
        ///    環境をクリーンアップします。
        /// </summary>
        [TestCleanup]
        public void TestCleanup()
        {
            //  フィールドをクリーンアップします。
            dataSet.Dispose();

            //  データベースをクリーンアップします。
            DataBaseMaintainer.Cleanup();
        }

        /// <summary>
        ///    Clear()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void Clear()
        {
            {
                //  利用者1のダミー・データがあることを確認します。
                ReservedItem.FillByUserInformationNumber(dataSet, userInformationNumber1);
                Assert.AreEqual(2, dataTable.Count);
            }

            dataSet.Clear();

            {
                //  利用者2のダミー・データがあることを確認します。
                ReservedItem.FillByUserInformationNumber(dataSet, userInformationNumber2);
                Assert.AreEqual(2, dataTable.Count);
            }

            ReservedItem.Clear(userInformationNumber1);

            dataSet.Clear();

            {
                //  利用者1のダミー・データが削除されたことを確認します。
                ReservedItem.FillByUserInformationNumber(dataSet, userInformationNumber1);
                Assert.AreEqual(0, dataTable.Count);
            }

            dataSet.Clear();

            {
                //  利用者2のダミー・データは削除されていないことを確認します。
                ReservedItem.FillByUserInformationNumber(dataSet, userInformationNumber2);
                Assert.AreEqual(2, dataTable.Count);
            }
        }

        /// <summary>
        ///    Reserve()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void Reserve()
        {
            {
                //  利用者1のダミー・データがあることを確認します。
                ReservedItem.FillByUserInformationNumber(dataSet, userInformationNumber1);
                Assert.AreEqual(1, ((RetailDataSet.ReservedItemRow)dataTable.Select("ItemIdentity = 1")[0]).Quantity);

                //  ダミー・データによって予約が設定されていることを確認します。
                Assert.AreEqual(2, ReservedItem.CalculateNotExpireQuantity(1, userInformationNumber3));
            }

            dataSet.Clear();

            {
                //  予約できることを確認します。
                ReservedItem.Reserve(1L, userInformationNumber1, 2);

                //  予約数量が書き換わっていることを確認します。
                ReservedItem.FillByUserInformationNumber(dataSet, userInformationNumber1);
                Assert.AreEqual(2, ((RetailDataSet.ReservedItemRow)dataTable.Select("ItemIdentity = 1")[0]).Quantity);

                //  予約が増えていることを確認します。
                Assert.AreEqual(3, ReservedItem.CalculateNotExpireQuantity(1, userInformationNumber3));
            }

            dataSet.Clear();

            {
                //  予約可能な数量を超えた場合は、予約可能な数量だけ予約できることを確認します。
                Assert.AreEqual(9, ReservedItem.Reserve(1L, userInformationNumber1, 10));

                //  予約が増えていることを確認します。
                Assert.AreEqual(10, ReservedItem.CalculateNotExpireQuantity(1, userInformationNumber3));
            }
        }

        /// <summary>
        ///    CalculateNotExpiredQuantity()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void CalculateNotExpireQuantity()
        {
            //  自分の予約は数に入らないことを確認します。
            Assert.AreEqual(1, ReservedItem.CalculateNotExpireQuantity(1, userInformationNumber1));

            //  他の利用者の予約がすべて数えられていることを確認します。
            Assert.AreEqual(2, ReservedItem.CalculateNotExpireQuantity(1, userInformationNumber3));
        }

        /// <summary>
        ///    入力値検証をテストします。
        /// </summary>
        [TestMethod]
        public void Validate()
        {
            //  追加よりも変更の方が楽なので、まずはデータを取得します。
            ReservedItem.Fill(dataSet);

            //  不正な入力値を作成。それぞれが独立であることが分かっているので、組み合わせのテストはしません。
            dataTable[0].Quantity = 0;
            dataTable[0].ExpireDateTime = DateTime.Now.AddMinutes(-10);

            //  Update()メソッドを呼びます。Validatorが呼び出され、入力値検証が実行されます。
            ReservedItem.Update(dataSet);

            //  正しくエラーが設定されたか検証します。
            Assert.AreEqual("予約数量は正の数でなければなりません。", dataTable[0].GetColumnError(dataTable.QuantityColumn));
            Assert.AreEqual("有効期限には未来の日付が設定されていなければなりません。", dataTable[0].GetColumnError(dataTable.ExpireDateTimeColumn));
        }

        /// <summary>
        ///    Fill()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void Fill()
        {
            //  Fill()メソッドを呼び出せることを確認します。
            ReservedItem.Fill(dataSet);
        }

        /// <summary>
        ///    FillByItemIdentityAndUserInformationNumber()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillByItemIdentityAndUserInformationNumber()
        {
            //  FillByItemIdentityAndUserInformationNumber()メソッドを呼び出せることを確認します。
            ReservedItem.FillByItemIdentityAndUserInformationNumber(dataSet, 1, new Guid("25b60925-a865-4ca8-90fe-6c6ed493e851"));
        }

        /// <summary>
        ///    FillByUserInformationNumber()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillByUserInformationNumber()
        {
            //  FillByUserInformationNumber()メソッドを呼び出せることを確認します。
            ReservedItem.FillByUserInformationNumber(dataSet, new Guid("25b60925-a865-4ca8-90fe-6c6ed493e851"));
        }

        /// <summary>
        ///    FillWithChildren()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillWithChildren()
        {
            //  FillWithChildren()メソッドを呼び出せることを確認します。
            ReservedItem.FillWithChildren(dataSet);
        }

        /// <summary>
        ///    GetData()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void GetData()
        {
            //  GetData()メソッドを呼び出せることを確認します。
            ReservedItem.GetData();
        }
    }
}
