﻿using System;
using System.Transactions;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail
{
    /// <summary>
    ///    品目に関するビジネス・ロジックを担当するクラスです。
    /// </summary>
    public static class Item
    {
        /// <summary>
        ///    商品識別子を指定して、予約されていない在庫数量を取得します。
        /// </summary>
        /// <param name="itemIdentity">商品識別子。</param>
        /// <returns>在庫数量</returns>
        public static int CalculateNotReservedQuantity(long itemIdentity, Guid userInformationNumber)
        {
            int? q = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            using (RetailDataSetTableAdapters.ItemTableAdapter ta = new RetailDataSetTableAdapters.ItemTableAdapter())
                            {
                                q = ta.CalculateQuantity(itemIdentity) - ReservedItem.CalculateNotExpireQuantity(itemIdentity, userInformationNumber);
                            }
                        });
                });

            return q.HasValue ? (q.Value > 0 ? q.Value : 0) : 0;
        }

        /// <summary>
        ///    指定した商品識別子を持つ商品の在庫数量を減らします。
        /// </summary>
        /// <param name="itemIdentity">商品識別子。</param>
        /// <param name="decreasingQuantity">在庫数量を減らす数。</param>
        public static void DecreaseQuantity(RetailDataSet dataSet, long itemIdentity, int count)
        {
            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            //  品目だけを更新対象にする（注文明細を更新対象から外す）ために、新しい型付DataSetを作成。
                            using (RetailDataSet ds = new RetailDataSet())
                            {
                                //  品目を取得。
                                FillByItemIdentity(ds, itemIdentity);

                                //  在庫数量を設定。
                                ds.Item[0].Quantity -= count;

                                //  更新。
                                Update(ds);

                                //  元の型付DataSetにマージ。
                                dataSet.Merge(ds);
                            }
                        }, dataSet);
                });
        }

        /// <summary>
        ///    品目を更新します。
        /// </summary>
        /// <param name="dataSet">更新データが格納された型付DataSet。</param>
        /// <returns>更新された品目の数。</returns>
        public static int Update(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // 入力データを検証。
                            RXForBusiness.CallValidators(dataSet.Item);

                            // 検証の結果、エラーが発見されたら……
                            if (dataSet.HasErrors)
                            {
                                // 更新せずにリターン。
                                return;
                            }

                            // データベースを更新。
                            r = RXForBusiness.CallUpdateTableAdapters(dataSet.Item);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    品目を取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <returns>取得した品目の数。</returns>
        public static int Fill(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdapters(dataSet.Item, string.Empty);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    商品識別子を指定して、品目を取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <param name="productIdentity">商品識別子。</param>
        /// <returns>取得した品目の数。</returns>
        public static int FillByProductIdentity(RetailDataSet dataSet, long productIdentity)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdapters(dataSet.Item, "ByProductIdentity", productIdentity);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    品目識別子を指定して、品目を取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <param name="itemIdentity">品目識別子。</param>
        /// <returns>取得した品目の数。</returns>
        public static int FillByItemIdentity(RetailDataSet dataSet, long itemIdentity)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdapters(dataSet.Item, "ByItemIdentity", itemIdentity);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    品目とその子テーブルを取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <returns>取得した品目の数。</returns>
        public static int FillWithChildren(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdaptersWithChildren(dataSet.Item, string.Empty);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    品目のみを取得します。
        /// </summary>
        /// <returns>取得した品目が格納された型付DataTable。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024")]
        public static RetailDataSet.ItemDataTable GetData()
        {
            RetailDataSet.ItemDataTable dataTable = null;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // 対象テーブルのみを取得。
                            using (RetailDataSetTableAdapters.ItemTableAdapter ta = new RetailDataSetTableAdapters.ItemTableAdapter())
                            {
                                dataTable = ta.GetData();
                            }
                        });
                });

            return dataTable;
        }
    }
}
