﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Text;
using System.Transactions;
using TailIsland.RX;
using TailIsland.PetShop.EmulatorServiceAgent;

namespace TailIsland.PetShop.Retail
{
    /// <summary>
    ///    注文に関するビジネス・ロジックを担当するクラスです。
    /// </summary>
    public static class Order
    {
        /// <summary>
        ///    ランダム。
        /// </summary>
        private static Random random = new Random();

        /// <summary>
        ///     注文します。
        /// </summary>
        /// <param name="dataSet">注文データが格納された型付DataSet。</param>
        public static void MakeOrder(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            //  すべての新規注文に対して……
                            foreach (RetailDataSet.OrderRow dr in RXForValidator.GetAddedRows(dataSet.Order))
                            {
                                //  注文番号を発番。
                                RXForBusiness.SetIdentity(dr, dataSet.Order.OrderNumberColumn, CreateOrderNumber());
                            }

                            //  すべての追加/変更された注文に対して……
                            foreach (RetailDataSet.OrderRow dr in RXForValidator.GetAddedOrModifiedRows(dataSet.Order))
                            {
                                //  予約。
                                Reserve(dataSet, dr.OrderNumber);

                                //  エラーが発見されたら……
                                if (dataSet.HasErrors)
                                {
                                    // リターン。
                                    return;
                                }

                                //  在庫割り当て。
                                DecreaseItemQuantity(dataSet, dr.OrderNumber);

                                //  エラーが発見されたら……
                                if (dataSet.HasErrors)
                                {
                                    // リターン。
                                    return;
                                }

                                //  クレジット・カードを承認し、クレジット・カード・チャージ番号を設定。
                                dr.CreditCardChargeNumber = AuthorizeCreditCard(dr.CreditCardCompany, dr.CreditCardNumber, dr.CreditCardExpireMonth, dr.CreditCardExpireYear, GetTotalPrice(dataSet, dr.OrderNumber));
                            }

                            //  予約を削除するために、追加/変更された注文を退避しておきます。
                            List<RetailDataSet.OrderRow> l = new List<RetailDataSet.OrderRow>();
                            foreach (RetailDataSet.OrderRow dr in RXForBusiness.GetAddedOrModifiedRows(dataSet.Order))
                            {
                                l.Add(dr);
                            }

                            //  更新。
                            Update(dataSet);

                            //  すべての追加/変更された注文に対して……
                            foreach (RetailDataSet.OrderRow dr in l)
                            {
                                //  予約。
                                ReservedItem.Clear(dr.UserInformationNumber);
                            }
                        },
                        dataSet);
                });
        }

        /// <summary>
        ///     注文番号を生成します。
        /// </summary>
        /// <returns>生成した注文番号。</returns>
        private static string CreateOrderNumber()
        {
            string s;

            do
            {
                //  注文番号をメモリ内で生成。
                s = CreateOrderNumberInMemory();
            }
            //  RDBMSに同じ注文番号がある場合はやり直し。
            while (!CheckSameOrderNumberIsNotInRdbms(s));

            return s;
        }

        /// <summary>
        ///     メモリ内で新しい注文番号を生成します。
        /// </summary>
        /// <returns>生成された売り上げ番号。</returns>
        private static string CreateOrderNumberInMemory()
        {
            StringBuilder sb = new StringBuilder();

            for (int i = 0; i < 3; ++i)
            {
                sb.Append(SelectCharRandomly("ABCDEFGHJKLMNPQRSTUVWXYZ"));
            }
            for (int i = 0; i < 4; ++i)
            {
                sb.Append(SelectCharRandomly("23456789"));
            }

            return sb.ToString();
        }

        /// <summary>
        ///    指定された文字列から、ランダムに一文字選択します。
        /// </summary>
        /// <param name="enableChars">選択対象の文字を含む文字列。</param>
        /// <returns>ランダムに選択された文字。</returns>
        private static char SelectCharRandomly(string chars)
        {
            lock (random)
            {
                return chars[random.Next(chars.Length)];
            }
        }

        /// <summary>
        ///     RDBMSに同じ注文番号がないことを確認します。
        /// </summary>
        /// <param name="ordeerNumber">注文番号。</param>
        /// <returns>同じ注文番号がなければtrue、そうでなければfalse。</returns>
        private static bool CheckSameOrderNumberIsNotInRdbms(string orderNumber)
        {
            try
            {
                // 競合を避けるために、RequiresNewで新しいトランザクションを開始。
                RXForBusiness.ExecuteTransaction(
                    delegate
                    {
                        //  データの挿入に成功したら、重複がないことになります。
                        using (RetailDataSetTableAdapters.OrderNumberTableAdapter ta = new RetailDataSetTableAdapters.OrderNumberTableAdapter())
                        {
                            ta.Insert(orderNumber);
                        }
                    },
                    null, TransactionScopeOption.RequiresNew);

                return true;
            }
            catch (SqlException ex)
            {
                if (ex.Number == 2627)
                {
                    return false;
                }

                throw;
            }
        }

        /// <summary>
        ///    クレジット・カードを承認します。
        /// </summary>
        /// <param name="company">クレジット・カード会社。</param>
        /// <param name="number">クレジット・カード番号。</param>
        /// <param name="expireMonth">クレジット・カード有効期限（月）</param>
        /// <param name="expireYear">クレジット・カード有効期限（年）</param>
        /// <returns>クレジット・カード・チャージ番号。承認できなかった場合は空文字列になります。</returns>
        private static string AuthorizeCreditCard(string company, string number, int expireMonth, int expireYear, decimal price)
        {
            string s;

            try
            {
                //  クレジット・カード承認のサービス・エージェントを生成。
                using (CreditCardAuthorizer ws = new CreditCardAuthorizer())
                {
                    //  サービス・エージェントの初期化。
                    RXForBusiness.InitializeServiceAgent(ws);

                    //  クレジット・カードを承認し、クレジット・カード・チャージ番号を取得。
                    s = ws.AuthorizeCreditCard(company, number, expireMonth, expireYear, price);
                }
            }
            catch (Exception ex)
            {
                Trace.TraceInformation(ex.ToString());

                throw new RXException("申し訳ありません。クレジット・カード認証サーバとの通信に失敗しました。しばらく時間をおいてから、もう一度リクエストを送信してください。");
            }

            return s;
        }

        /// <summary>
        ///    品目の在庫数量を減らし、在庫を割り当てます。
        /// </summary>
        /// <param name="dataSet">型付DataSet。</param>
        /// <param name="orderNumber">在庫数量を減らす注文の注文番号。</param>
        private static void DecreaseItemQuantity(RetailDataSet dataSet, string orderNumber)
        {
            foreach (RetailDataSet.OrderDetailRow dr in dataSet.Order.FindByOrderNumber(orderNumber).GetOrderDetailRows())
            {
                Item.DecreaseQuantity(dataSet, dr.ItemRow.Identity, dr.Quantity);
            }
        }

        /// <summary>
        ///    予約します。
        /// </summary>
        /// <param name="dataSet">予約する注文が格納された型付DataSet。</param>
        /// <param name="orderNumber">予約する注文の注文識別子。</param>
        public static void Reserve(RetailDataSet dataSet, string orderNumber)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            foreach (RetailDataSet.OrderDetailRow dr in dataSet.Order.FindByOrderNumber(orderNumber).GetOrderDetailRows())
                            {
                                OrderDetail.Reserve((RetailDataSet)dr.Table.DataSet, dr.Identity);
                            }
                        }, dataSet);
                });
        }

        /// <summary>
        ///    注文明細の合計金額を計算します。
        /// </summary>
        /// <param name="dataSet">計算対象が格納された型付DataSet。</param>
        /// <param name="orderNumber">合計金額を計算する注文の注文番号。</param>
        /// <returns>合計金額。</returns>
        public static decimal GetTotalPrice(RetailDataSet dataSet, string orderNumber)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            decimal r = 0M;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            foreach (RetailDataSet.OrderDetailRow dr in dataSet.Order.FindByOrderNumber(orderNumber).GetOrderDetailRows())
                            {
                                r += dr.Price;
                            }
                        });
                });

            return r;
        }

        /// <summary>
        ///    注文を更新します。
        /// </summary>
        /// <param name="dataSet">更新データが格納された型付DataSet。</param>
        /// <returns>更新された注文の数。</returns>
        public static int Update(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // 入力データを検証。
                            RXForBusiness.CallValidators(dataSet.Order);

                            // 検証の結果、エラーが発見されたら……
                            if (dataSet.HasErrors)
                            {
                                // 更新せずにリターン。
                                return;
                            }

                            // データベースを更新。
                            r = RXForBusiness.CallUpdateTableAdapters(dataSet.Order);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    注文を取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <returns>取得した注文の数。</returns>
        public static int Fill(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdapters(dataSet.Order, string.Empty);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    注文とその子テーブルを取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <returns>取得した注文の数。</returns>
        public static int FillWithChildren(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdaptersWithChildren(dataSet.Order, string.Empty);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    注文のみを取得します。
        /// </summary>
        /// <returns>取得した注文が格納された型付DataTable。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024")]
        public static RetailDataSet.OrderDataTable GetData()
        {
            RetailDataSet.OrderDataTable dataTable = null;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // 対象テーブルのみを取得。
                            using (RetailDataSetTableAdapters.OrderTableAdapter ta = new RetailDataSetTableAdapters.OrderTableAdapter())
                            {
                                dataTable = ta.GetData();
                            }
                        });
                });

            return dataTable;
        }
    }
}
