﻿using System;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail
{
    /// <summary>
    ///    注文明細に関するビジネス・ロジックを担当するクラスです。
    /// </summary>
    public static class OrderDetail
    {
        /// <summary>
        ///    予約します。
        /// </summary>
        /// <param name="dataSet">予約するデータが格納された型付DataSet。</param>
        /// <param name="orderDetailIdentity">注文明細識別子。</param>
        /// <returns>予約に成功した場合はtrue。予約に失敗した場合はfalse。なお、注文明細の購入数量は、予約できた数量で上書きされます。</returns>
        public static void Reserve(RetailDataSet dataSet, long orderDetailIdentity)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            //  注文明細を取得。
                            RetailDataSet.OrderDetailRow dr = dataSet.OrderDetail.FindByIdentity(orderDetailIdentity);

                            //  予約。
                            int q = ReservedItem.Reserve(dr.ItemIdentity, dr.OrderRow.UserInformationNumber, dr.Quantity);

                            //  予約できた数量が購入する数量と異なる場合は……
                            if (q != dr.Quantity)
                            {
                                //  購入数量を調整。
                                dr.Quantity = q;

                                //  エラーを設定。
                                RXForBusiness.AddColumnError(dr, dataSet.OrderDetail.QuantityColumn, q > 0 ?
                                    string.Format("申し訳ありません。{0}（{1}）は、数量{2}しか在庫がありません。", dr.ItemRow.ProductRow.Name, dr.ItemRow.Name, q) :
                                    string.Format("申し訳ありません。{0}（{1}）は、在庫がありません。", dr.ItemRow.ProductRow.Name, dr.ItemRow.Name));
                            }
                        },
                        dataSet);
                });
        }

        /// <summary>
        ///    注文明細を更新します。
        /// </summary>
        /// <param name="dataSet">更新データが格納された型付DataSet。</param>
        /// <returns>更新された注文明細の数。</returns>
        public static int Update(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // 入力データを検証。
                            RXForBusiness.CallValidators(dataSet.OrderDetail);

                            // 検証の結果、エラーが発見されたら……
                            if (dataSet.HasErrors)
                            {
                                // 更新せずにリターン。
                                return;
                            }

                            // データベースを更新。
                            r = RXForBusiness.CallUpdateTableAdapters(dataSet.OrderDetail);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    注文明細を取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <returns>取得した注文明細の数。</returns>
        public static int Fill(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdapters(dataSet.OrderDetail, string.Empty);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    注文明細とその子テーブルを取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <returns>取得した注文明細の数。</returns>
        public static int FillWithChildren(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdaptersWithChildren(dataSet.OrderDetail, string.Empty);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    注文明細のみを取得します。
        /// </summary>
        /// <returns>取得した注文明細が格納された型付DataTable。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024")]
        public static RetailDataSet.OrderDetailDataTable GetData()
        {
            RetailDataSet.OrderDetailDataTable dataTable = null;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // 対象テーブルのみを取得。
                            using (RetailDataSetTableAdapters.OrderDetailTableAdapter ta = new RetailDataSetTableAdapters.OrderDetailTableAdapter())
                            {
                                dataTable = ta.GetData();
                            }
                        });
                });

            return dataTable;
        }
    }
}
