﻿using System;
using System.Data;
using System.Transactions;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail
{
    /// <summary>
    ///    予約済品目に関するビジネス・ロジックを担当するクラスです。
    /// </summary>
    public static class ReservedItem
    {
        /// <summary>
        ///    予約します。
        /// </summary>
        /// <param name="itemIdentity">品目識別子。</param>
        /// <param name="userInformationNumber">ユーザ番号。</param>
        /// <param name="quantity">予約数量。</param>
        /// <returns>予約できた数量。</returns>
        public static int Reserve(long itemIdentity, Guid userInformationNumber, int quantity)
        {
            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。競合の発生を防ぐために、RequiresNewで別トランザクションにします。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            //  楽観的同時実行制御で失敗した場合にはリトライ。
                            RXForBusiness.RetryWhenDBConcurrencyException(
                                delegate
                                {
                                    using (RetailDataSet ds = new RetailDataSet())
                                    {
                                        //  予約済品目を取得。
                                        FillByItemIdentityAndUserInformationNumber(ds, itemIdentity, userInformationNumber);

                                        //  予約済品目が存在しない場合は……
                                        if (ds.ReservedItem.Count == 0)
                                        {
                                            //  予約済品目を作成。
                                            ds.ReservedItem.AddReservedItemRow(
                                                itemIdentity,
                                                userInformationNumber,
                                                quantity,
                                                DateTime.Now.AddMinutes(10));
                                        }
                                        //  予約済品目が既に存在している場合は……
                                        else
                                        {
                                            //  数量と有効期限を設定。
                                            ds.ReservedItem[0].Quantity = quantity;
                                            ds.ReservedItem[0].ExpireDateTime = DateTime.Now.AddMinutes(10);
                                        }

                                        //  在庫数量に合わせて予約数量を調整。
                                        AdjustQuantity(ds.ReservedItem[0]);

                                        //  調整済みの予約数量を戻り値に設定。
                                        r = ds.ReservedItem[0].Quantity;

                                        //  更新。
                                        Update(ds);
                                    }
                                });
                        },
                        null, TransactionScopeOption.RequiresNew);
                });

            return r;
        }

        /// <summary>
        ///    在庫数量に合わせて予約数量を調整します。
        /// </summary>
        /// <param name="dataRow">予約済品目が格納された型付DataRow。</param>
        /// <returns>予約可能な数量。</returns>
        private static void AdjustQuantity(RetailDataSet.ReservedItemRow dataRow)
        {
            //  まだ予約されていない在庫数量を取得。
            int q = Item.CalculateNotReservedQuantity(dataRow.ItemIdentity, dataRow.UserInformationNumber);

            //  予約数量を調整。
            if (q < dataRow.Quantity)
            {
                dataRow.Quantity = q;
            }
        }

        /// <summary>
        ///    予約を破棄します。
        /// </summary>
        /// <param name="userInformationNumber">予約を破棄するユーザのユーザ番号</param>
        public static void Clear(Guid userInformationNumber)
        {
            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。競合の発生を防ぐために、RequiresNewで別トランザクションにします。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            //  楽観的同時実行制御で失敗した場合にはリトライ。
                            RXForBusiness.RetryWhenDBConcurrencyException(
                                delegate
                                {
                                    using (RetailDataSet ds = new RetailDataSet())
                                    {
                                        //  予約済品目を取得。
                                        FillByUserInformationNumber(ds, userInformationNumber);

                                        //  すべてのレコードを削除。
                                        foreach (RetailDataSet.ReservedItemRow dr in ds.ReservedItem)
                                        {
                                            dr.Delete();
                                        }

                                        //  更新。
                                        Update(ds);
                                    }
                                });
                        },
                        null);
                });
        }

        /// <summary>
        ///    品目識別子を指定して、ユーザ番号が異なって予約破棄時刻が現在よりも後の予約済品目の数量を計算します。
        /// </summary>
        /// <param name="itemIdentity">品目識別子。</param>
        /// <param name="exclugingUserInformationNumber">ユーザ番号。このユーザ番号の予約は計算結果に含まれません。</param>
        /// <returns>予約済みの数量。</returns>
        public static int CalculateNotExpireQuantity(long itemIdentity, Guid exclugingUserInformationNumber)
        {
            int? r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            using (RetailDataSetTableAdapters.ReservedItemTableAdapter ta = new RetailDataSetTableAdapters.ReservedItemTableAdapter())
                            {
                                r = ta.CalculateNotExpireQuantity(itemIdentity, exclugingUserInformationNumber);
                            }
                        });
                });

            return r.HasValue ? (r.Value > 0 ? r.Value : 0) : 0;
        }

        /// <summary>
        ///    予約済品目を更新します。
        /// </summary>
        /// <param name="dataSet">更新データが格納された型付DataSet。</param>
        /// <returns>更新された予約済品目の数。</returns>
        public static int Update(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // 入力データを検証。
                            RXForBusiness.CallValidators(dataSet.ReservedItem);

                            // 検証の結果、エラーが発見されたら……
                            if (dataSet.HasErrors)
                            {
                                // 更新せずにリターン。
                                return;
                            }

                            // データベースを更新。
                            r = RXForBusiness.CallUpdateTableAdapters(dataSet.ReservedItem);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    予約済品目を取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <returns>取得した予約済品目の数。</returns>
        public static int Fill(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdapters(dataSet.ReservedItem, string.Empty);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    品目識別子とユーザ番号を指定して、予約済品目を取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <param name="itemIdentity">品目識別子。</param>
        /// <param name="userInformationNumber">ユーザ番号。</param>
        /// <returns>取得した予約済品目の数。</returns>
        public static int FillByItemIdentityAndUserInformationNumber(RetailDataSet dataSet, long itemIdentity, Guid userInformationNumber)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdapters(dataSet.ReservedItem, "ByItemIdentityAndUserInformationNumber", itemIdentity, userInformationNumber);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    ユーザ番号を指定して、予約済品目を取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <param name="userInformationNumber">ユーザ番号。</param>
        /// <returns>取得した予約済品目の数。</returns>
        public static int FillByUserInformationNumber(RetailDataSet dataSet, Guid userInformationNumber)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdapters(dataSet.ReservedItem, "ByUserInformationNumber", userInformationNumber);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    予約済品目とその子テーブルを取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <returns>取得した予約済品目の数。</returns>
        public static int FillWithChildren(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdaptersWithChildren(dataSet.ReservedItem, string.Empty);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    予約済品目のみを取得します。
        /// </summary>
        /// <returns>取得した予約済品目が格納された型付DataTable。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024")]
        public static RetailDataSet.ReservedItemDataTable GetData()
        {
            RetailDataSet.ReservedItemDataTable dataTable = null;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // 対象テーブルのみを取得。
                            using (RetailDataSetTableAdapters.ReservedItemTableAdapter ta = new RetailDataSetTableAdapters.ReservedItemTableAdapter())
                            {
                                dataTable = ta.GetData();
                            }
                        });
                });

            return dataTable;
        }
    }
}
