﻿using System;
using System.Web.Security;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail
{
    /// <summary>
    ///    ユーザに関するビジネス・ロジックを担当するクラスです。
    /// </summary>
    public static class UserInformation
    {
        /// <summary>
        ///     システムとデータベースにユーザを登録します。
        /// </summary>
        /// <param name="userName">ユーザ名。</param>
        /// <param name="password">パスワード。</param>
        /// <param name="eMail">Eメール・アドレス。</param>
        /// <param name="passwordQuestion">セキュリティの質問。</param>
        /// <param name="passwordAnswer">セキュリティの解答。</param>
        /// <param name="zipCode">郵便番号。</param>
        /// <param name="address1">住所1。</param>
        /// <param name="address2">住所2。</param>
        /// <param name="phoneNumber">電話番号。</param>
        /// <param name="name">氏名。</param>
        public static void CreateUserInformation(string userName, string password, string eMail, string passwordQuestion, string passwordAnswer, string zipCode, string address1, string address2, string phoneNumber, string name)
        {
            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            MembershipCreateStatus mcs;
                            MembershipUser mu = Membership.CreateUser(userName, password, eMail, passwordQuestion, passwordAnswer, true, out mcs);

                            if (mu == null)
                            {
                                throw new MembershipCreateUserException(mcs);
                            }

                            Roles.AddUserToRole(userName, "users");

                            using (RetailDataSet ds = new RetailDataSet())
                            {
                                ds.UserInformation.AddUserInformationRow((Guid)mu.ProviderUserKey, zipCode, address1, address2, phoneNumber, name);

                                Update(ds);

                                //  DataSetを返せないので、エラーが発生した場合は例外をスローします。
                                if (ds.HasErrors)
                                {
                                    throw new MembershipCreateUserException(RXForBusiness.CollectDataSetErrors(ds));
                                }
                            }
                        });
                });
        }

        /// <summary>
        ///    ユーザを更新します。
        /// </summary>
        /// <param name="dataSet">更新データが格納された型付DataSet。</param>
        /// <returns>更新されたユーザの数。</returns>
        public static int Update(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // 入力データを検証。
                            RXForBusiness.CallValidators(dataSet.UserInformation);

                            // 検証の結果、エラーが発見されたら……
                            if (dataSet.HasErrors)
                            {
                                // 更新せずにリターン。
                                return;
                            }

                            // データベースを更新。
                            r = RXForBusiness.CallUpdateTableAdapters(dataSet.UserInformation);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    ユーザを取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <returns>取得したユーザの数。</returns>
        public static int Fill(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdapters(dataSet.UserInformation, string.Empty);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    ユーザ番号を指定して、ユーザを取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <param name="userInformationNumber">ユーザ番号。</param>
        /// <returns>取得したユーザの数。</returns>
        public static int FillByUserInformationNumber(RetailDataSet dataSet, Guid userInformationNumber)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdapters(dataSet.UserInformation, "ByUserInformationNumber", userInformationNumber);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    ユーザとその子テーブルを取得します。
        /// </summary>
        /// <param name="dataSet">取得したデータを格納する型付DataSet。</param>
        /// <returns>取得したユーザの数。</returns>
        public static int FillWithChildren(RetailDataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            int r = 0;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // データベースを検索。
                            r = RXForBusiness.CallFillTableAdaptersWithChildren(dataSet.UserInformation, string.Empty);
                        },
                        dataSet);
                });

            return r;
        }

        /// <summary>
        ///    ユーザのみを取得します。
        /// </summary>
        /// <returns>取得したユーザが格納された型付DataTable。</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024")]
        public static RetailDataSet.UserInformationDataTable GetData()
        {
            RetailDataSet.UserInformationDataTable dataTable = null;

            //  トレース。
            RXForBusiness.TraceInAndOut(
                delegate
                {
                    // トランザクションを開始。
                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            // 対象テーブルのみを取得。
                            using (RetailDataSetTableAdapters.UserInformationTableAdapter ta = new RetailDataSetTableAdapters.UserInformationTableAdapter())
                            {
                                dataTable = ta.GetData();
                            }
                        });
                });

            return dataTable;
        }
    }
}
