﻿using System;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Threading;
using System.Transactions;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace TailIsland.RX.Test
{
    [TestClass]
    public class RXForBusinessUnitTest
    {
        private static TraceListener rdbmsTraceListener;

        [ClassInitialize]
        public static void ClassInitialize(TestContext testContext)
        {
            if (AlreadyExistRdbmsTraceListener())
            {
                return;
            }

            rdbmsTraceListener = new RdbmsTraceListener();
            rdbmsTraceListener.Filter = new EventTypeFilter(SourceLevels.Error | SourceLevels.Warning | SourceLevels.Information | SourceLevels.Verbose);
            rdbmsTraceListener.TraceOutputOptions = TraceOptions.DateTime | TraceOptions.Timestamp | TraceOptions.ProcessId | TraceOptions.ThreadId;

            Trace.Listeners.Add(rdbmsTraceListener);
        }

        private static bool AlreadyExistRdbmsTraceListener()
        {
            foreach (TraceListener tl in Trace.Listeners)
            {
                if (tl is RdbmsTraceListener)
                {
                    return true;
                }
            }

            return false;
        }

        [TestInitialize]
        public void TestInitialize()
        {
            RXForUnitTest.ExecuteCreateSqlScript("PetShop", "TailIsland.RX");
            RXForUnitTest.ExecuteChangeSqlScript("PetShop", "TailIsland.RX");
        }

        [TestCleanup]
        public void TestCleanup()
        {
            RXForUnitTest.ExecuteCreateSqlScript("PetShop", "TailIsland.RX-drop");
        }

        [ClassCleanup]
        public static void ClassCleanup()
        {
            if (rdbmsTraceListener == null)
            {
                return;
            }

            Trace.Listeners.Remove(rdbmsTraceListener);
        }

        [TestMethod]
        public void TraceInAndOut()
        {
            RXForBusiness.TraceInAndOut(
                delegate()
                {
                    Thread.Sleep(1000);
                },
                "RX", 1);

            using (SqlConnection scon = new SqlConnection(ConfigurationManager.ConnectionStrings["RXConnectionString"].ConnectionString))
            {
                scon.Open();

                using (SqlCommand scom = new SqlCommand("select [Type], [Message] from [Trace]", scon))
                {
                    using (SqlDataReader sdr = scom.ExecuteReader())
                    {
                        sdr.Read();

                        Assert.AreEqual("Information", sdr.GetString(0));
                        Assert.AreEqual("RXForBusinessUnitTest.TraceInAndOutが開始しました。\r\n  パラメータ1: RX\r\n  パラメータ2: 1", sdr.GetString(1));

                        sdr.Read();

                        Assert.AreEqual("Information", sdr.GetString(0));
                        Assert.AreEqual("RXForBusinessUnitTest.TraceInAndOutが終了しました。\r\n  パラメータ1: RX\r\n  パラメータ2: 1", sdr.GetString(1));
                    }
                }
            }
        }

        [TestMethod]
        public void ExecuteTransaction()
        {
            //  トランザクションが生成されていることをテストします。
            
            Assert.IsNull(Transaction.Current);

            RXForBusiness.ExecuteTransaction(
                delegate
                {
                    Assert.IsNotNull(Transaction.Current);
                });

            //  トランザクションがコミットされることをテストします。
            
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.ExecuteTransaction(
                    delegate
                    {
                        RXForBusiness.CallFillTableAdaptersWithChildren(ds.T01, string.Empty);
                        
                        ds.T01[0].Value = "Changed";

                        RXForBusiness.CallUpdateTableAdapters(ds.T01);
                    });
            }

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdapters(ds.T01, string.Empty);
                Assert.AreEqual("Changed", ds.T01[0].Value);
            }

            //  トランザクションがロールバックされることをテストします。

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.ExecuteTransaction(
                    delegate
                    {
                        RXForBusiness.CallFillTableAdaptersWithChildren(ds.T02, string.Empty);

                        ds.T02[0].Value = "Changed";
                        
                        RXForBusiness.CallUpdateTableAdapters(ds.T02);

                        ds.T02[0].SetColumnError(ds.T02.ValueColumn, "X");
                    }, 
                    ds);
            }

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdapters(ds.T02, string.Empty);
                Assert.AreNotEqual("Changed", ds.T02[0].Value);
            }

            //  トランザクション・スコープ・オプションが設定できることをテストします。

            RXForBusiness.ExecuteTransaction(
                delegate
                {
                    Transaction t = Transaction.Current;

                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            Assert.IsTrue(Transaction.Current == t);
                        });
                });

            RXForBusiness.ExecuteTransaction(
                delegate
                {
                    Transaction t = Transaction.Current;

                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            Assert.IsTrue(Transaction.Current != t);
                        }, 
                        null, TransactionScopeOption.RequiresNew);
                });

            //  トランザクションがアボートされた場合に、処理が実行されないことをテストします。

            RXForBusiness.ExecuteTransaction(
                delegate
                {
                    bool isExecuted = false;

                    using (RXDataSet ds = new RXDataSet())
                    {
                        RXForBusiness.ExecuteTransaction(
                            delegate
                            {
                                ds.T03.AddT03Row(0L, "X").SetColumnError(ds.T03.ValueColumn, "X");
                            }, 
                            ds);
                    }

                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            isExecuted = true;
                        });

                    Assert.IsFalse(isExecuted);
                });

            //  トランザクションがアボートされなければ、処理が実行されることをテストします。

            RXForBusiness.ExecuteTransaction(
                delegate
                {
                    bool isExecuted = false;

                    using (RXDataSet ds = new RXDataSet())
                    {
                        RXForBusiness.ExecuteTransaction(
                            delegate
                            {
                                ds.Reset();
                            }, 
                            ds);
                    }

                    RXForBusiness.ExecuteTransaction(
                        delegate
                        {
                            isExecuted = true;
                        });

                    Assert.IsTrue(isExecuted);
                });
        }

        [TestMethod]
        public void CallFillTableAdaptersForUpdate()
        {
            using (RXDataSet ds = new RXDataSet())
            {
                Assert.AreEqual(3, RXForBusiness.CallFillTableAdaptersWithChildren(ds.T04, string.Empty));

                AssertFilledDataTableIsCollect(ds.T01, 2);
                AssertFilledDataTableIsCollect(ds.T02, 2);
                AssertFilledDataTableIsCollect(ds.T03, 2);
                AssertFilledDataTableIsCollect(ds.T04, 3);
                AssertFilledDataTableIsCollect(ds.T05, 0);
                AssertFilledDataTableIsCollect(ds.T06, 2);
                AssertFilledDataTableIsCollect(ds.T07, 2);
                AssertFilledDataTableIsCollect(ds.T08, 2);
                AssertFilledDataTableIsCollect(ds.T09, 0);
                AssertFilledDataTableIsCollect(ds.T10, 3);
            }
        }

        [TestMethod]
        public void CallFillTableAdaptersForRead()
        {
            using (RXDataSet ds = new RXDataSet())
            {
                Assert.AreEqual(3, RXForBusiness.CallFillTableAdapters(ds.T04, string.Empty));

                AssertFilledDataTableIsCollect(ds.T01, 2);
                AssertFilledDataTableIsCollect(ds.T02, 2);
                AssertFilledDataTableIsCollect(ds.T03, 2);
                AssertFilledDataTableIsCollect(ds.T04, 3);
                AssertFilledDataTableIsCollect(ds.T05, 0);
                AssertFilledDataTableIsCollect(ds.T06, 0);
                AssertFilledDataTableIsCollect(ds.T07, 0);
                AssertFilledDataTableIsCollect(ds.T08, 0);
                AssertFilledDataTableIsCollect(ds.T09, 0);
                AssertFilledDataTableIsCollect(ds.T10, 0);
            }

            //  FillTableAdapterCaller.maxLength以下では例外が出ないことを確認します。

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdapters(ds.T01, string.Empty);

                for (long l = ds.T01.Count; l < (long)TailIsland_RX_FillTableAdaptersCallerAccessor.maxRecordCount; ++l)
                {
                    ds.T01.AddT01Row(l, ds.T01[0], "X");
                }
                
                RXForBusiness.CallUpdateTableAdapters(ds.T01);
            }

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdapters(ds.T04, string.Empty);
            }

            //  FillTableAdapterCaller.maxLength行で例外が出ることを確認します。

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T01, string.Empty);

                ds.T01.AddT01Row((long)TailIsland_RX_FillTableAdaptersCallerAccessor.maxRecordCount, ds.T01[0], "X");
                
                RXForBusiness.CallUpdateTableAdapters(ds.T01);
            }

            using (RXDataSet ds = new RXDataSet())
            {
                try
                {
                    RXForBusiness.CallFillTableAdapters(ds.T04, string.Empty);

                    Assert.Fail();
                }
                catch (ArgumentException ex) 
                {
                    Assert.AreEqual("取得したレコード数が多すぎます。テーブルT01から501レコードを取得しました。", ex.Message);
                }
            }
        }

        private static void AssertFilledDataTableIsCollect(DataTable dataTable, int rowCount)
        {
            Assert.AreEqual(rowCount, dataTable.Rows.Count);

            for (int i = 0; i < rowCount; ++i)
            {
                Assert.AreEqual(dataTable.Rows[i]["Value"], string.Format("{0}-{1}", dataTable.TableName, i));
            }
        }

        [TestMethod]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Maintainability", "CA1502")]
        public void CallUpdateTableAdapters()
        {
            //  Identityが再セットされることを確認します。

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T04, string.Empty);

                RXDataSet.T04Row t04r1 = ds.T04.AddT04Row(100L, ds.T02[0], ds.T03[0], "a");
                RXDataSet.T07Row t07r1 = ds.T07.AddT07Row(100L, t04r1, "b");
                RXDataSet.T07Row t07r2 = ds.T07.AddT07Row(101L, t04r1, "c");
                RXDataSet.T08Row t08r1 = ds.T08.AddT08Row(100L, t04r1, ds.T06[0], "d");
                RXDataSet.T10Row t10r1 = ds.T10.AddT10Row(100L, t07r1, ds.T10[0], "e");
                RXDataSet.T10Row t10r2 = ds.T10.AddT10Row(101L, t07r1, t10r1, "f");

                RXForBusiness.CallUpdateTableAdapters(ds.T04);
            }

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T04, string.Empty);

                RXDataSet.T04Row t04r1 = ds.T04[3];
                Assert.AreEqual(1001L, t04r1.Identity);
                Assert.AreEqual("a", t04r1.Value);

                RXDataSet.T07Row t07r1 = (RXDataSet.T07Row)t04r1.GetChildRows(ds.T04.ChildRelations["T07_T04_FK"])[0];
                Assert.AreEqual(1001L, t07r1.Identity);
                Assert.AreEqual("b", t07r1.Value);

                RXDataSet.T07Row t07r2 = (RXDataSet.T07Row)t04r1.GetChildRows(ds.T04.ChildRelations["T07_T04_FK"])[1];
                Assert.AreEqual(1002L, t07r2.Identity);
                Assert.AreEqual("c", t07r2.Value);

                RXDataSet.T08Row t08r1 = (RXDataSet.T08Row)t04r1.GetChildRows(ds.T04.ChildRelations["T08_T04_FK"])[0];
                Assert.AreEqual(1001L, t08r1.Identity);
                Assert.AreEqual("d", t08r1.Value);

                RXDataSet.T10Row t10r1 = (RXDataSet.T10Row)t07r1.GetChildRows(ds.T07.ChildRelations["T10_T07_FK"])[0];
                Assert.AreEqual(1001L, t10r1.Identity);
                Assert.AreEqual("e", t10r1.Value);

                RXDataSet.T10Row t10r2 = (RXDataSet.T10Row)t07r1.GetChildRows(ds.T07.ChildRelations["T10_T07_FK"])[1];
                Assert.AreEqual(1002L, t10r2.Identity);
                Assert.AreEqual("f", t10r2.Value);
            }

            //  Create + Create.

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                ds.T11.AddT11Row(2L, "a");
                ds.T12.AddT12Row(2L, 2L, ds.T11[2], "a");
                
                Assert.AreEqual(1, RXForBusiness.CallUpdateTableAdapters(ds.T11));
            }
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                Assert.AreEqual(3, ds.T11.Rows.Count);
                Assert.AreEqual(3, ds.T12.Rows.Count);

                Assert.AreEqual("a", ds.T11[2].Value);
                Assert.AreEqual("a", ds.T12[2].Value);
            }

            //  Create + None.

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                ds.T11.AddT11Row(3L, "b");

                Assert.AreEqual(1, RXForBusiness.CallUpdateTableAdapters(ds.T11));
            }
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                Assert.AreEqual(4, ds.T11.Rows.Count);
                Assert.AreEqual(3, ds.T12.Rows.Count);

                Assert.AreEqual("b", ds.T11[3].Value);
            }

            //  Update + Create.

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                ds.T11[3].Value = "c";
                ds.T12.AddT12Row(3L, 3L, ds.T11[3], "c");

                Assert.AreEqual(1, RXForBusiness.CallUpdateTableAdapters(ds.T11));
            }
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                Assert.AreEqual(4, ds.T11.Rows.Count);
                Assert.AreEqual(4, ds.T12.Rows.Count);

                Assert.AreEqual("c", ds.T11[3].Value);
                Assert.AreEqual("c", ds.T12[3].Value);
            }

            //  Update + Update.

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                ds.T11[3].Value = "d";
                ds.T12[3].Value = "d";

                Assert.AreEqual(1, RXForBusiness.CallUpdateTableAdapters(ds.T11));
            }
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                Assert.AreEqual(4, ds.T11.Rows.Count);
                Assert.AreEqual(4, ds.T12.Rows.Count);

                Assert.AreEqual("d", ds.T11[3].Value);
                Assert.AreEqual("d", ds.T12[3].Value);
            }

            //  Update + Delete.

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                ds.T11[3].Value = "e";
                ds.T12[3].Delete();

                Assert.AreEqual(1, RXForBusiness.CallUpdateTableAdapters(ds.T11));
            }
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                Assert.AreEqual(4, ds.T11.Rows.Count);
                Assert.AreEqual(3, ds.T12.Rows.Count);

                Assert.AreEqual("e", ds.T11[3].Value);
            }

            //  Update + None.

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                ds.T11[3].Value = "e";

                Assert.AreEqual(1, RXForBusiness.CallUpdateTableAdapters(ds.T11));
            }
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                Assert.AreEqual(4, ds.T11.Rows.Count);
                Assert.AreEqual(3, ds.T12.Rows.Count);

                Assert.AreEqual("e", ds.T11[3].Value);
            }

            //  Delete + Delete.

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                ds.T11[2].Delete();
                ds.T12[2].Delete();

                Assert.AreEqual(1, RXForBusiness.CallUpdateTableAdapters(ds.T11));
            }
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                Assert.AreEqual(3, ds.T11.Rows.Count);
                Assert.AreEqual(2, ds.T12.Rows.Count);
            }

            //  Delete + None.

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                ds.T11[2].Delete();

                Assert.AreEqual(1, RXForBusiness.CallUpdateTableAdapters(ds.T11));
            }
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                Assert.AreEqual(2, ds.T11.Rows.Count);
                Assert.AreEqual(2, ds.T12.Rows.Count);
            }

            //  None + Create.

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                ds.T12.AddT12Row(2L, 2L, ds.T11[1], "f");

                Assert.AreEqual(0, RXForBusiness.CallUpdateTableAdapters(ds.T11));
            }
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                Assert.AreEqual(2, ds.T11.Rows.Count);
                Assert.AreEqual(3, ds.T12.Rows.Count);

                Assert.AreEqual("f", ds.T12[2].Value);
            }

            //  None + Update.

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                ds.T12[2].Value = "g";

                Assert.AreEqual(0, RXForBusiness.CallUpdateTableAdapters(ds.T11));
            }
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                Assert.AreEqual(2, ds.T11.Rows.Count);
                Assert.AreEqual(3, ds.T12.Rows.Count);

                Assert.AreEqual("g", ds.T12[2].Value);
            }

            //  None + Delete.

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                ds.T12[2].Delete();

                Assert.AreEqual(0, RXForBusiness.CallUpdateTableAdapters(ds.T11));
            }
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                Assert.AreEqual(2, ds.T11.Rows.Count);
                Assert.AreEqual(2, ds.T12.Rows.Count);
            }

            //  None + None.

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);
                Assert.AreEqual(0, RXForBusiness.CallUpdateTableAdapters(ds.T11));
            }
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                Assert.AreEqual(2, ds.T11.Rows.Count);
                Assert.AreEqual(2, ds.T12.Rows.Count);
            }
        }

        [TestMethod]
        public void CallValidators()
        {
            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                RXForValidator.AddRowError(ds.T11[1], "a");
                RXForValidator.AddRowError(ds.T12[1], "b");

                Assert.AreEqual("a", ds.T11[1].RowError);
                Assert.AreEqual("b", ds.T12[1].RowError);

                ds.T11[0].Value = "Changed";
                ds.T12[0].Value = "Changed";

                RXForBusiness.CallValidators(ds.T11);

                Assert.AreEqual(string.Empty, ds.T11[1].RowError);
                Assert.AreEqual(string.Empty, ds.T12[1].RowError);

                Assert.AreEqual("X", ds.T11[0].RowError);
                Assert.AreEqual("Y", ds.T12[0].RowError);
            }

            using (RXDataSet ds = new RXDataSet())
            {
                RXForBusiness.CallFillTableAdaptersWithChildren(ds.T11, string.Empty);

                ds.T11[0].Value = "Changed";
                ds.T12[0].Value = "Changed";
                
                RXForBusiness.CallValidators(ds.T11);

                Assert.AreEqual(string.Empty, ds.T11[1].RowError);
                Assert.AreEqual(string.Empty, ds.T12[1].RowError);

                Assert.AreEqual("X", ds.T11[0].RowError);
                Assert.AreEqual("Y", ds.T12[0].RowError);
            }
        }
    }
}
