﻿using System;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Text;
using System.Threading;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace TailIsland.RX.Test
{
    [TestClass]
    public class RdbmsTraceListenerUnitTest
    {
        private static TraceListener rdbmsTraceListener;

        [ClassInitialize]
        public static void ClassInitialize(TestContext testContext)
        {
            if (AlreadyExistRdbmsTraceListener())
            {
                return;
            }

            rdbmsTraceListener = new RdbmsTraceListener();
            rdbmsTraceListener.Filter = new EventTypeFilter(SourceLevels.Error | SourceLevels.Warning | SourceLevels.Information | SourceLevels.Verbose);
            rdbmsTraceListener.TraceOutputOptions = TraceOptions.DateTime | TraceOptions.Timestamp | TraceOptions.ProcessId | TraceOptions.ThreadId;

            Trace.Listeners.Add(rdbmsTraceListener);
        }

        private static bool AlreadyExistRdbmsTraceListener()
        {
            foreach (TraceListener tl in Trace.Listeners)
            {
                if (tl is RdbmsTraceListener)
                {
                    return true;
                }
            }

            return false;
        }

        [TestInitialize]
        public void TestInitialize()
        {
            RXForUnitTest.ExecuteCreateSqlScript("PetShop", "TailIsland.RX");
            RXForUnitTest.ExecuteChangeSqlScript("PetShop", "TailIsland.RX");
        }

        [TestCleanup]
        public void TestCleanup()
        {
            RXForUnitTest.ExecuteCreateSqlScript("PetShop", "TailIsland.RX-drop");
        }

        [ClassCleanup]
        public static void ClassCleanup()
        {
            if (rdbmsTraceListener == null)
            {
                return;
            }

            Trace.Listeners.Remove(rdbmsTraceListener);
        }

        [TestMethod]
        public void TraceError()
        {
            Trace.TraceError(Create2KString());

            CheckTraceRowWithNotNull("Error", Create1KString());
        }

        [TestMethod]
        public void TraceWarning()
        {
            Trace.TraceWarning(Create2KString());

            CheckTraceRowWithNotNull("Warning", Create1KString());
        }

        [TestMethod]
        public void TraceInformation()
        {
            Trace.TraceInformation(Create2KString());

            CheckTraceRowWithNotNull("Information", Create1KString());
        }

        [TestMethod]
        public void Write()
        {
            Trace.Write(Create2KString());

            CheckTraceRowWithNull("Verbose", Create1KString());
        }

        [TestMethod]
        public void WriteLine()
        {
            Trace.Write(Create2KString());

            CheckTraceRowWithNull("Verbose", Create1KString());
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Security", "CA2122")]
        private static void CheckTraceRowWithNotNull(string type, string message)
        {
            using (SqlConnection scon = new SqlConnection(ConfigurationManager.ConnectionStrings["RXConnectionString"].ConnectionString))
            {
                scon.Open();

                using (SqlCommand scom = new SqlCommand("select [DateTime], [Timestamp], [MachineName], [ProcessId], [ThreadId], [Type], [Message] from [Trace]", scon))
                {
                    using (SqlDataReader sdr = scom.ExecuteReader())
                    {
                        sdr.Read();

                        Assert.IsTrue(sdr.GetDateTime(0) <= DateTime.Now);
                        Assert.IsTrue(sdr.GetInt64(1) <= Stopwatch.GetTimestamp());
                        Assert.AreEqual(Environment.MachineName, sdr.GetString(2));
                        Assert.AreEqual(Process.GetCurrentProcess().Id, sdr.GetInt32(3));
                        Assert.AreEqual(Convert.ToString(Thread.CurrentThread.ManagedThreadId), sdr.GetString(4));
                        Assert.AreEqual(type, sdr.GetString(5));
                        Assert.AreEqual(message, sdr.GetString(6));
                    }
                }
            }
        }

        private static void CheckTraceRowWithNull(string type, string message)
        {
            using (SqlConnection scon = new SqlConnection(ConfigurationManager.ConnectionStrings["RXConnectionString"].ConnectionString))
            {
                scon.Open();

                using (SqlCommand scom = new SqlCommand("select [DateTime], [Timestamp], [MachineName], [ProcessId], [ThreadId], [Type], [Message] from [Trace]", scon))
                {
                    using (SqlDataReader sdr = scom.ExecuteReader())
                    {
                        sdr.Read();

                        Assert.IsTrue(sdr.IsDBNull(0));
                        Assert.IsTrue(sdr.IsDBNull(1));
                        Assert.IsTrue(sdr.IsDBNull(2));
                        Assert.IsTrue(sdr.IsDBNull(3));
                        Assert.IsTrue(sdr.IsDBNull(4));
                        Assert.AreEqual(type, sdr.GetString(5));
                        Assert.AreEqual(message, sdr.GetString(6));
                    }
                }
            }
        }

        private static string Create1KString()
        {
            StringBuilder sb = new StringBuilder();

            for (int i = 0; i < 1024; ++i)
            {
                sb.Append((char)('0' + i % 10));
            }

            return sb.ToString();
        }

        private static string Create2KString()
        {
            StringBuilder sb = new StringBuilder();

            for (int i = 0; i < 2048; ++i)
            {
                sb.Append((char)('0' + i % 10));
            }

            return sb.ToString();
        }
    }
}
