﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Transactions;
using System.Web.Services.Protocols;

namespace TailIsland.RX
{
    public static class RXForBusiness
    {
        public static void TraceInAndOut(Procedure procedure, params object[] args)
        {
            if (procedure == null)
            {
                throw new ArgumentNullException("procedure");
            }

            new InOutTracer(procedure, args).TraceInOut();
        }

        public static void ExecuteTransaction(Procedure transactionProcedure)
        {
            ExecuteTransaction(transactionProcedure, null);
        }

        public static void ExecuteTransaction(Procedure transactionProcedure, DataSet consistentFlagDataSet)
        {
            ExecuteTransaction(transactionProcedure, consistentFlagDataSet, TransactionScopeOption.Required);
        }

        public static void ExecuteTransaction(Procedure transactionProcedure, DataSet consistentFlagDataSet, TransactionScopeOption transactionScopeOption)
        {
            if (transactionProcedure == null)
            {
                throw new ArgumentNullException("transactionProcedure");
            }

            new TransactionExecutor(transactionProcedure, consistentFlagDataSet, transactionScopeOption).ExecuteTransaction();
        }

        public static void RetryWhenDBConcurrencyException(Procedure procedure)
        {
            if (procedure == null)
            {
                throw new ArgumentNullException("procedure");
            }

            new WhenDBConcurrencyExceptionRetrier(procedure).RetryWhenDBConcurrencyException();
        }

        public static int CallFillTableAdapters(DataTable dataTable, string condition, params object[] extraParameters)
        {
            if (dataTable == null)
            {
                throw new ArgumentNullException("dataTable");
            }
            if (condition == null)
            {
                throw new ArgumentNullException("condition");
            }
            if (extraParameters == null)
            {
                throw new ArgumentNullException("extraParameters");
            }

            return new FillTableAdaptersCaller(dataTable, null, condition, extraParameters).CallFillTableAdapters();
        }

        public static int CallFillTableAdaptersWithChildren(DataTable dataTable, string condition, params object[] extraParameters)
        {
            return CallFillTableAdaptersWithChildren(dataTable, null, condition, extraParameters);
        }

        public static int CallFillTableAdaptersWithChildren(DataTable dataTable, ICollection<DataTable> lastChildDataTables, string condition, params object[] extraParameters)
        {
            if (dataTable == null)
            {
                throw new ArgumentNullException("dataTable");
            }
            if (condition == null)
            {
                throw new ArgumentNullException("condition");
            }
            if (extraParameters == null)
            {
                throw new ArgumentNullException("extraParameters");
            }

            return new FillTableAdaptersCaller(dataTable, lastChildDataTables, condition, extraParameters).CallFillTableAdaptersWithChildren();
        }

        public static void CallValidators(DataTable dataTable, params object[] extraParameters)
        {
            if (dataTable == null)
            {
                throw new ArgumentNullException("dataTable");
            }
            if (extraParameters == null)
            {
                throw new ArgumentNullException("extraParameters");
            }

            new ValidatorsCaller(dataTable, extraParameters).CallValidators();
        }

        public static int CallUpdateTableAdapters(DataTable dataTable)
        {
            if (dataTable == null)
            {
                throw new ArgumentNullException("dataTable");
            }

            return new UpdateTableAdaptersCaller(dataTable).CallUpdateTableAdapters();
        }

        public static IEnumerable<DataRow> GetAddedOrModifiedRows(DataTable dataTable)
        {
            if (dataTable == null)
            {
                throw new ArgumentNullException("dataTable");
            }

            return new AddedOrModifiedRowsGetter(dataTable).GetAddedOrModifiedRows();
        }

        public static IEnumerable<DataRow> GetAddedRows(DataTable dataTable)
        {
            if (dataTable == null)
            {
                throw new ArgumentNullException("dataTable");
            }

            return new AddedRowsGetter(dataTable).GetAddedRows();
        }

        public static long GenerateNewIdentity(string tableName)
        {
            if (tableName == null)
            {
                throw new ArgumentNullException("tableName");
            }

            return new NewIdentityGenerator(tableName).GenerateNewIdentity();
        }

        public static void SetIdentity(DataRow dataRow, DataColumn dataColumn, object identity)
        {
            if (dataRow == null)
            {
                throw new ArgumentNullException("dataRow");
            }
            if (dataColumn == null)
            {
                throw new ArgumentNullException("dataColumn");
            }
            if (identity == null)
            {
                throw new ArgumentNullException("identity");
            }

            new IdentitySetter(dataRow, dataColumn, identity).SetIdentity();
        }

        public static void ClearDataSetErrors(DataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            new DataSetErrorsClearer(dataSet).ClearErrors();
        }

        public static string CollectDataSetErrors(DataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            return new DataSetErrorsCollector(dataSet).CollectDataSetErrors().Replace(System.Environment.NewLine, "<br />");
        }

        public static void AddRowError(DataRow dataRow, string errorMessage)
        {
            if (dataRow == null)
            {
                throw new ArgumentNullException("dataRow");
            }

            new RowErrorAdder(dataRow, errorMessage, null).AddRowError();
        }

        public static void AddRowError(DataRow dataRow, string errorMessage, Function<bool> hasError)
        {
            if (dataRow == null)
            {
                throw new ArgumentNullException("dataRow");
            }

            new RowErrorAdder(dataRow, errorMessage, hasError).AddRowError();
        }

        public static void AddColumnError(DataRow dataRow, DataColumn dataColumn, string errorMessage)
        {
            if (dataRow == null)
            {
                throw new ArgumentNullException("dataRow");
            }
            if (dataColumn == null)
            {
                throw new ArgumentNullException("dataColumn");
            }

            new ColumnErrorAdder(dataRow, dataColumn, errorMessage, null).AddColumnError();
        }

        public static void AddColumnError(DataRow dataRow, DataColumn dataColumn, string errorMessage, Function<bool> hasError)
        {
            if (dataRow == null)
            {
                throw new ArgumentNullException("dataRow");
            }
            if (dataColumn == null)
            {
                throw new ArgumentNullException("dataColumn");
            }

            new ColumnErrorAdder(dataRow, dataColumn, errorMessage, hasError).AddColumnError();
        }

        public static void InitializeServiceAgent(SoapHttpClientProtocol soapHttpClientProtocol)
        {
            if (soapHttpClientProtocol == null)
            {
                throw new ArgumentNullException("soapHttpClientProtocol");
            }

            new ServiceAgentInitializer(soapHttpClientProtocol).InitializeServiceAgent();
        }
    }
}
