﻿using System;
using System.Diagnostics;
using System.Text;
using System.Web;
using System.Web.Services.Protocols;
using System.Xml;

namespace TailIsland.RX
{
    public static class RXForGlobalAsax
    {
        private const long maxSessionLength = 500L * 1024L;
        private const long maxResponseLength = 200L * 1024L;
        private const int maxDuration = 8 * 1000;

        public static void OnStart()
        {
            Trace.Listeners.Add(CreateRdbmsTraceListener());
        }

        private static TraceListener CreateRdbmsTraceListener()
        {
            TraceListener tl = new RdbmsTraceListener();

            tl.Name = "rdbmsTraceListener";
            tl.TraceOutputOptions = TraceOptions.DateTime | TraceOptions.Timestamp | TraceOptions.ProcessId | TraceOptions.ThreadId;

#if DEBUG
            tl.Filter = new EventTypeFilter(SourceLevels.Error | SourceLevels.Warning | SourceLevels.Information | SourceLevels.Verbose);
#else
            tl.Filter = new EventTypeFilter(SourceLevels.Error | SourceLevels.Warning | SourceLevels.Information);
#endif

            return tl;
        }

        public static void OnEnd()
        {
            Trace.Listeners.Remove("rdbmsTraceListener");
        }

        public static void PreRequestHandlerExecute()
        {
            if (IsAspDotNetRequest())
            {
#if DEBUG
                HttpContext.Current.Items["RXForGlobalAsax.StartDateTime"] = DateTime.Now;
                HttpContext.Current.Response.Filter = new ResponseLengthCalculatorStream(HttpContext.Current.Response.Filter);
#endif

                string s;
                if (!new JisX0208Checker(IsAspxRequest() ? GetPostParameters() : GetSoapParameters()).CheckJisX0208(out s))
                {
                    RespondError(string.Format("申し訳ありません。「{0}」は環境依存文字なので使用できません。", s));
                    return;
                }
            }
        }

        private static string GetPostParameters()
        {
            StringBuilder sb = new StringBuilder();

            foreach (string s in HttpContext.Current.Request.Params)
            {
                sb.Append(HttpContext.Current.Request.Params[s]);
            }

            return sb.ToString();
        }

        private static string GetSoapParameters()
        {
            try
            {
                StringBuilder sb = new StringBuilder();

                HttpContext.Current.Request.InputStream.Position = 0;
                using (XmlReader xr = XmlReader.Create(HttpContext.Current.Request.InputStream))
                {
                    while (xr.Read())
                    {
                        sb.Append(xr.Value);
                    }
                }

                return sb.ToString();
            }
            catch (XmlException)
            {
                return string.Empty;
            }
            finally
            {
                HttpContext.Current.Request.InputStream.Position = 0;
            }
        }

        public static void PostRequestHandlerExecute()
        {
            if (IsAspDotNetRequest())
            {
#if DEBUG
                if ((DateTime.Now - (DateTime)HttpContext.Current.Items["RXForGlobalAsax.StartDateTime"]).TotalMilliseconds > maxDuration)
                {
                    RespondError(string.Format("滞留時間が{0}秒を超えました。", maxDuration / 1000));
                    return;
                }

                if (HttpContext.Current.Response.Filter.Length > maxResponseLength)
                {
                    RespondError(string.Format("レスポンスのサイズが{0}KByteを超えました。", maxResponseLength / 1024));
                    return;
                }

                if (new SessionLengthCalculator(HttpContext.Current.Session).CalculateSessionLength() > maxSessionLength)
                {
                    RespondError(string.Format("セッションのサイズが{0}KByteを超えました。", maxSessionLength / 1024));
                    return;
                }
#endif
            }
        }

        private static void RespondError(string message)
        {
            if (IsAspxRequest())
            {
                HttpContext.Current.Server.Transfer(string.Format("~/Error.aspx?errorMessage={0}", HttpUtility.UrlEncode(message)), false);
            }

            if (IsAsmxRequest())
            {
                HttpContext.Current.Response.Clear();
                HttpContext.Current.Response.AddHeader("Content-Type", new SoapContentTypeCreator(GetSoapProtocolVersion(), GetSoapEncoding()).CreateSoapContentType());
                HttpContext.Current.Response.Output.Write(
                    new SoapFaultCreator(GetSoapProtocolVersion(), GetSoapEncoding(), new SoapExceptionCreator(message).CreateSoapException()).CreateSoapFault());
                HttpContext.Current.Response.End();
            }
        }

        private static SoapProtocolVersion GetSoapProtocolVersion()
        {
            try
            {
                HttpContext.Current.Request.InputStream.Position = 0;
                using (XmlReader xr = XmlReader.Create(HttpContext.Current.Request.InputStream))
                {
                    while (xr.Read())
                    {
                        if (xr.LocalName == "Envelope")
                        {
                            return xr.NamespaceURI == "http://schemas.xmlsoap.org/soap/envelope/" ? SoapProtocolVersion.Soap11 : SoapProtocolVersion.Soap12;
                        }
                    }
                }

                return SoapProtocolVersion.Default;
            }
            catch (XmlException)
            {
                return SoapProtocolVersion.Default;
            }
            finally
            {
                HttpContext.Current.Request.InputStream.Position = 0;
            }
        }

        private static Encoding GetSoapEncoding()
        {
            return HttpContext.Current.Response.HeaderEncoding;
        }

        private static bool IsAspDotNetRequest()
        {
            return IsAspxRequest() || IsAsmxRequest();
        }

        private static bool IsAspxRequest()
        {
            return HttpContext.Current.Request.Path.ToLower().EndsWith(".aspx");
        }

        private static bool IsAsmxRequest()
        {
            return HttpContext.Current.Request.Path.ToLower().EndsWith(".asmx");
        }

        //  何故だか分りませんけど、イベントに+=したのではイベント・ハンドラが呼び出さされませんでした……。
        //  Errorだけはイベント・ハンドラが呼び出されるんですけど、複数回+=すると複数回呼び出されちゃうし……。
    }
}
