﻿using System;
using System.Text;
using System.Web.Services.Protocols;
using System.Xml;

namespace TailIsland.RX
{
    internal sealed class SoapFaultCreator
    {
        private SoapException soapException;
        private SoapProtocolVersion soapProtocolVersion;
        private Encoding encoding;

        internal SoapFaultCreator(SoapProtocolVersion soapProtocolVersion, Encoding encoding, SoapException soapException)
        {
            this.soapException = soapException;
            this.soapProtocolVersion = soapProtocolVersion;
            this.encoding = encoding;
        }

        internal string CreateSoapFault()
        {
            return soapProtocolVersion == SoapProtocolVersion.Soap11 ? CreateSoap11Fault() : CreateSoap12Fault();
        }

        private string CreateSoap11Fault()
        {
            const string prefix = "soap";
            const string ns = "http://schemas.xmlsoap.org/soap/envelope/";

            StringBuilder sb = new StringBuilder();

            using (XmlWriter xw = XmlWriter.Create(sb, CreateXmlWriterSettings(encoding)))
            {
                xw.WriteStartDocument();
                xw.WriteStartElement(prefix, "Envelope", ns);
                xw.WriteStartElement(prefix, "Body", ns);
                xw.WriteStartElement(prefix, "Fault", ns);
                xw.WriteStartElement("faultcode", string.Empty);
                xw.WriteQualifiedName(ConvertToSoap11FaultCode(soapException.Code).Name, ConvertToSoap11FaultCode(soapException.Code).Namespace);
                xw.WriteEndElement();
                xw.WriteStartElement("faultstring", string.Empty);
                xw.WriteAttributeString("xml", "lang", "http://www.w3.org/XML/1998/namespace", "ja");
                xw.WriteString(soapException.Message);
                xw.WriteEndElement();
                xw.WriteStartElement("faultactor", string.Empty);
                xw.WriteString(soapException.Actor);
                xw.WriteEndElement();
                xw.WriteStartElement("detail", string.Empty);
                if (soapException.Detail != null)
                {
                    soapException.Detail.WriteContentTo(xw);
                }
                xw.WriteEndElement();
                xw.WriteEndElement();
                xw.WriteEndElement();
                xw.WriteEndElement();
                xw.WriteEndDocument();
            }

            return sb.ToString();
        }

        private static XmlQualifiedName ConvertToSoap11FaultCode(XmlQualifiedName code)
        {
            if (code == Soap12FaultCodes.ReceiverFaultCode)
            {
                return SoapException.ServerFaultCode;
            }
            if (code == Soap12FaultCodes.SenderFaultCode)
            {
                return SoapException.ClientFaultCode;
            }
            if (code == Soap12FaultCodes.MustUnderstandFaultCode)
            {
                return SoapException.MustUnderstandFaultCode;
            }
            if (code == Soap12FaultCodes.VersionMismatchFaultCode)
            {
                return SoapException.VersionMismatchFaultCode;
            }

            return code;
        }

        private string CreateSoap12Fault()
        {
            const string prefix = "env";
            const string ns = "http://www.w3.org/2003/05/soap-envelope";

            StringBuilder sb = new StringBuilder();

            using (XmlWriter xw = XmlWriter.Create(sb, CreateXmlWriterSettings(encoding)))
            {
                xw.WriteStartDocument();
                xw.WriteStartElement(prefix, "Envelope", ns);
                xw.WriteStartElement(prefix, "Body", ns);
                xw.WriteStartElement(prefix, "Fault", ns);
                xw.WriteStartElement(prefix, "Code", ns);
                xw.WriteStartElement(prefix, "Value", ns);
                xw.WriteQualifiedName(ConvertToSoap12FaultCode(soapException.Code).Name, ConvertToSoap12FaultCode(soapException.Code).Namespace);
                if (soapException.SubCode != null)
                {
                    xw.WriteStartElement(prefix, "SubCode", ns);
                    xw.WriteStartElement(prefix, "Value", ns);
                    xw.WriteQualifiedName(soapException.SubCode.Code.Name, soapException.SubCode.Code.Namespace);
                    xw.WriteEndElement();
                    xw.WriteEndElement();
                }
                xw.WriteEndElement();
                xw.WriteEndElement();
                xw.WriteStartElement(prefix, "Reason", ns);
                xw.WriteStartElement(prefix, "Text", ns);
                xw.WriteAttributeString("xml", "lang", "http://www.w3.org/XML/1998/namespace", "ja");
                xw.WriteString(soapException.Message);
                xw.WriteEndElement();
                xw.WriteEndElement();
                xw.WriteStartElement(prefix, "Node", ns);
                xw.WriteString(soapException.Actor);
                xw.WriteEndElement();
                xw.WriteStartElement(prefix, "Role", ns);
                xw.WriteString(soapException.Role);
                xw.WriteEndElement();
                xw.WriteStartElement(prefix, "Detail", ns);
                if (soapException.Detail != null)
                {
                    soapException.Detail.WriteContentTo(xw);
                }
                xw.WriteEndElement();
                xw.WriteEndElement();
                xw.WriteEndElement();
                xw.WriteEndElement();
                xw.WriteEndDocument();
            }

            return sb.ToString();
        }

        private static XmlQualifiedName ConvertToSoap12FaultCode(XmlQualifiedName code)
        {
            if (code == SoapException.ServerFaultCode)
            {
                return Soap12FaultCodes.ReceiverFaultCode;
            }
            if (code == SoapException.ClientFaultCode)
            {
                return Soap12FaultCodes.SenderFaultCode;
            }
            if (code == SoapException.MustUnderstandFaultCode)
            {
                return Soap12FaultCodes.MustUnderstandFaultCode;
            }
            if (code == SoapException.VersionMismatchFaultCode)
            {
                return Soap12FaultCodes.VersionMismatchFaultCode;
            }

            return code;
        }

        private static XmlWriterSettings CreateXmlWriterSettings(Encoding encoding)
        {
            XmlWriterSettings xws = new XmlWriterSettings();

            xws.CloseOutput = false;
            xws.Encoding = encoding;

            return xws;
        }
    }
}
